#!/usr/bin/env python
# -*- coding: latin-1 -*-

"""HEN eye-tracking experiment"""

from psychopy import core, visual, gui, data, event
import numpy as np
import os, glob

# DEFINE USEFUL FUNCTIONS
def fileWrite(fileName, textList):
	# useful file writer function
	# takes filename and list as inputs
	fileHandler = open(fileName, 'a')
	thing = u''
	for item in textList:
		thing = thing + (unicode(item)+'\t')
	thing = thing + '\n'
	fileHandler.write(thing.encode('utf-8'))
	fileHandler.close()

# screen size
from win32api import GetSystemMetrics
scx = GetSystemMetrics (0)
scy = GetSystemMetrics (1)

# read files, sort by conditions and all that
# list files
DIR = os.path.dirname(os.path.realpath(__file__))  	#current dir
stimuliFiles = glob.glob(os.path.join(DIR,'cond_*.txt')) # all the instructions
dataDir = os.path.join(DIR, 'data') 		# save to this dir
instructionFiles = glob.glob(os.path.join(DIR,'instructions*.txt')) # all the instructions
practiceFile = glob.glob(os.path.join(DIR,'practice.txt')) # all the instructions

# QUERY PARTICIPANT 
# dialog boxes - who is the participant (:
subjectInfo = {'Nummer':000, u'lder':00, 'Eyetracking':1}
dlg = gui.DlgFromDict(subjectInfo, title=u'Lsexperiment')
if dlg.OK:
	outputFileName = str(subjectInfo['Nummer'])+'.txt'
	outputFileName = os.path.join(dataDir , outputFileName)
	ET = subjectInfo['Eyetracking']
	#create
	fileWrite(outputFileName, 
		['Number', 'Age', 'TrialNo','StimSet', 'SentenceNumber', 'Condition', 'RT_Read','ResponseQ', 'RT_Q','Correct_Response','Stimuli', 'Question']) 
		#headers (outputFileName, [subjectInfo[u'Nummer'], subjectInfo[u'lder'], trialNo, thisTxt[1], thisTxt[2], response[0][1], ans, response2[0][1], thisTxt[5], thisTxt[3] ])	
else:
    core.quit() #the user hit cancel so exit

# DO STUFF IN THE BEGINNING

# select conditions

thisCondition = subjectInfo['Nummer']%8  # this can be number between 0 and 7 (ie 8 possible stimulus configurations)

# select stimuli file, load, read
thisStimuli = stimuliFiles[thisCondition]
stimuliText = open(thisStimuli)
stimuliText= stimuliText.read().decode('latin-1')
stimuliText = [x for x in stimuliText.split('\n')]
np.random.shuffle(stimuliText)

# randomise which trials have follow up questions
## there are 48 critical trial 50% Q
criticalQ = [0]*24 + [1]*24
np.random.shuffle(criticalQ)
## there are 40 non critical trials, 50% Q
fillerQ = [0]*20 + [1]*20
np.random.shuffle(fillerQ)

# open window
w = visual.Window([scx,scy], fullscr = True, allowGUI= False	, monitor='testMonitor', units='pix', color = [205, 205, 205], colorSpace = 'rgb255')			# window

# define stimuli
theFix = visual.Rect(w, pos=(-550,160), height = 20, width = 20, lineColor = "black", lineWidth=6)
theInstruction = visual.TextStim(w, pos = (0, 0), text = '', wrapWidth = 600, height = 28, color = 'black', alignVert= 'center', font = "Courier New") ###this is the instruction text
theStatement = visual.TextStim(w, text = "", pos = (-550, 160), height = 40, color = "black", alignVert = "top", alignHoriz = "left", 
	wrapWidth = scx - 200, font = "Courier New") ## thisis the statement/main stimlus
theQ = visual.TextStim(w, text = "", pos = (0, 100), height = 40, color = "black", alignVert = "top", alignHoriz = "center", 
	wrapWidth = scx - 200, font = "Courier New") ## thisis the statement/main stimlus
theFollowupInstruction = visual.TextStim(w, pos = (0, -150), text = u'Tryck VNSTER fr NEJ eller HGER fr JA', wrapWidth = 750, height = 25, color = 'black', alignVert= 'top', font = "Courier New") ###this is the instruction text

# show stimuli
critCount = 0
fillerCount = 0
trialNo = 0
response2 = []
nLetters = 45
nLines = 3
#
# 1 INSTRUCTIONS
#
for i in range(len(instructionFiles)):
	instructionText = open(instructionFiles[i]).read().decode('latin-1')
	theInstruction.setText(instructionText)
	theInstruction.draw()
	w.flip()
	core.wait(1) # safety
	event.waitKeys(keyList = ['f2', 'space'])

#
# 2 EXAMPLES 
#

# load, read
practiceText = open(practiceFile[0])
practiceText= practiceText.read().decode('latin-1')
practiceText = [x for x in practiceText.split('\n')]

for prac in practiceText:
	# parse stim
	thisTxt = [x for x in prac.split('\t')]
	sT = thisTxt[3].split(' ')
	nWords = len((sT))
		
	# show fix
	theFix.draw()
	w.flip()
	core.wait(3)
	
	# show stim
	count = 0
	first = 1
	line = ''
	for i in range(nWords):
		noend = 1
		test = line + ' ' + sT[i]
		if first:
			test = test[1:]
			first = 0 
		if len(test) >= nLetters:
			line = '\n'*(count*nLines) + line
			theStatement.setText(line)
			theStatement.draw()
			line = sT[i]
			count += 1
			noend = 0
		else:
			line = test
	if noend:
		line = '\n'*(count*nLines) + line
		theStatement.setText(line)
		theStatement.draw()
	w.flip()
	core.wait(0.5)
	response = event.waitKeys(keyList = ['escape', 'space'])
	if response[0][0] == 'escape': #get me out of here
		core.quit()
	
	if thisTxt[2]=="A":
		followQ = 1
	else:
		followQ = 0
	
	if followQ:
		theQ.setText(thisTxt[4])
		theQ.draw()
		theFollowupInstruction.draw()
		w.flip()
		core.wait(0.5)
		response2 = event.waitKeys(keyList = ['f', 'j'])

#
# CALIBRATION
#

if ET:
	w.close()
	
	from iViewXAPI import  *            #iViewX library
	from iViewXAPIReturnCodes import * 
	# run calibration
	# ---------------------------------------------
	#---- connect to iViewX 
	# ---------------------------------------------
	
	res = iViewXAPI.iV_SetLogger(c_int(1), c_char_p("iViewXSDK_Python_Slideshow_Demo.txt"))
	res = iViewXAPI.iV_Connect(c_char_p('192.168.0.1'), c_int(4444), c_char_p('192.168.0.2'), c_int(5555))
	if res != 1:
		HandleError(res)
		exit(0)
		
	res = iViewXAPI.iV_GetSystemInfo(byref(systemData))
	print "iV_GetSystemInfo: " + str(res)
	print "Samplerate: " + str(systemData.samplerate)
	print "iViewX Version: " + str(systemData.iV_MajorVersion) + "." + str(systemData.iV_MinorVersion) + "." + str(systemData.iV_Buildnumber)
	print "iViewX API Version: " + str(systemData.API_MajorVersion) + "." + str(systemData.API_MinorVersion) + "." + str(systemData.API_Buildnumber)

	# ---------------------------------------------
	#---- configure and start calibration
	# ---------------------------------------------

	calibrationData = CCalibration(9, 1, 0, 0, 1, 250, 220, 2, 20, b"")

	res = iViewXAPI.iV_SetupCalibration(byref(calibrationData))
	print "iV_SetupCalibration " + str(res)

	while 1:
		res = iViewXAPI.iV_Calibrate()
		print "iV_Calibrate " + str(res)

		res = iViewXAPI.iV_Validate()
		print "iV_Validate " + str(res)

		res = iViewXAPI.iV_GetAccuracy(byref(accuracyData), 1)
		print "iV_GetAccuracy " + str(res)
		print "deviationXLeft " + str(accuracyData.deviationLX) + " deviationYLeft " + str(accuracyData.deviationLY)
		print "deviationXRight " + str(accuracyData.deviationRX) + " deviationYRight " + str(accuracyData.deviationRY)

		res = iViewXAPI.iV_ShowTrackingMonitor()
		print "iV_ShowTrackingMonitor " + str(res)
		
		q = raw_input("Calibration OK? (y/n) ")
		
		if q == "y":
			break

if ET:
	# open window
	w = visual.Window([scx,scy], fullscr = True, allowGUI= False	, monitor='testMonitor', units='pix', color = [205, 205, 205], colorSpace = 'rgb255')			# window

	# define stimuli
	theFix = visual.Rect(w, pos=(-550,160), height = 20, width = 20, lineColor = "black", lineWidth=6)
	theInstruction = visual.TextStim(w, pos = (0, 0), text = '', wrapWidth = 600, height = 28, color = 'black', alignVert= 'center', font = "Courier New") ###this is the instruction text
	theStatement = visual.TextStim(w, text = "", pos = (-550, 160), height = 40, color = "black", alignVert = "top", alignHoriz = "left", 
		wrapWidth = scx - 200, font = "Courier New") ## thisis the statement/main stimlus
	theQ = visual.TextStim(w, text = "", pos = (0, 100), height = 40, color = "black", alignVert = "top", alignHoriz = "center", 
		wrapWidth = scx - 200, font = "Courier New") ## thisis the statement/main stimlus
	theFollowupInstruction = visual.TextStim(w, pos = (0, -150), text = u'Tryck VNSTER fr NEJ eller HGER fr JA', wrapWidth = 750, height = 25, color = 'black', alignVert= 'top', font = "Courier New") ###this is the instruction text


#
# GET READY
#

theInstruction.setText(u'Tryck mellanslag fr att starta experimentet')
theInstruction.draw()
w.flip()
core.wait(0.5)
event.waitKeys(keyList = ['f2', 'space'])


#
# 3 MAIN LOOP
#
for stim in stimuliText:
	#stim = stimuliText[53] # debug
	# iterate trial
	trialNo = trialNo + 1
	
	# parse stim
	thisTxt = [x for x in stim.split('\t')]
	sT = thisTxt[3].split(' ')
	nWords = len((sT))
	#print(sT)
	#print(nWords)
	
	#show fix
	theFix.draw()
	w.flip()
	
	# start ET
	trialInfo = thisTxt[0] + '_' + thisTxt[2] + '.jpg'
	if ET:
		iViewXAPI.iV_StartRecording()
		iViewXAPI.iV_SendImageMessage(c_char_p(trialInfo))
	
	# wait
	core.wait(3)
	
	# show stim
	count = 0
	first = 1
	line = ''
	for i in range(nWords):
		noend = 1
		test = line + ' ' + sT[i]
		if first: # to remove the first space otherwise inserted
			test = test[1:]
			first = 0 
		if len(test) >= nLetters:
			line = '\n'*(count*nLines) + line
			theStatement.setText(line)
			theStatement.draw()
			line = sT[i]
			count += 1
			noend = 1
		else:
			line = test
	if noend:
		line = '\n'*(count*nLines) + line
		theStatement.setText(line)
		theStatement.draw()
	w.flip()
	timer = core.Clock()
	
	if ET:
		iViewXAPI.iV_SendImageMessage(c_char_p("Stimulus shown"))
	
	core.wait(0.5)
	response = event.waitKeys(keyList = ['escape', 'space'], timeStamped=timer)
	if response[0][0] == 'escape': #get me out of here
		core.quit()
	
	# stop ET
	if ET:
		iViewXAPI.iV_StopRecording()
	
	# if follow up Q, show
	if thisTxt[1]=="X":
		followQ = fillerQ[fillerCount]
		fillerCount += 1
	else:
		followQ = criticalQ[critCount]
		critCount += 1
	
	ans = "x"
	rt2 = 999
	
	if followQ:
		theQ.setText(thisTxt[4])
		theQ.draw()
		theFollowupInstruction.draw()
		w.flip()
		core.wait(0.5)
		timer = core.Clock()
		response2 = event.waitKeys(keyList = ['f', 'j'], timeStamped=timer)
		if response2[0][0] == "f":
			ans = "N"
		elif response2[0][0] == "j":
			ans = "Y"
		rt2 = response2[0][1]
	
	# save trial info
	print([subjectInfo[u'Nummer'], subjectInfo[u'lder'], trialNo, thisTxt[1], thisTxt[2], response[0][1], ans, rt2, thisTxt[5], thisTxt[3], thisTxt[4] ])
	fileWrite(outputFileName, [subjectInfo[u'Nummer'], subjectInfo[u'lder'], trialNo, thisTxt[0], thisTxt[2], response[0][1], ans, rt2, thisTxt[5], thisTxt[3], thisTxt[4] ])
	
	if trialNo == 44: # halfway through
		#
		# CALIBRATION
		#
		
		if ET:
			theInstruction.setText(u'Paus. Pkalla frsksledarens uppmrksamhet.')
			theInstruction.draw()
			w.flip()
			core.wait(2)
			event.waitKeys(keyList = ['f2', 'space'])
			
			w.close()
			
			from iViewXAPI import  *            #iViewX library
			from iViewXAPIReturnCodes import * 
			# run calibration
			# ---------------------------------------------
			#---- connect to iViewX 
			# ---------------------------------------------
			
			res = iViewXAPI.iV_SetLogger(c_int(1), c_char_p("iViewXSDK_Python_Slideshow_Demo.txt"))
			res = iViewXAPI.iV_Connect(c_char_p('192.168.0.1'), c_int(4444), c_char_p('192.168.0.2'), c_int(5555))
			if res != 1:
				HandleError(res)
				exit(0)
				
			res = iViewXAPI.iV_GetSystemInfo(byref(systemData))
			print "iV_GetSystemInfo: " + str(res)
			print "Samplerate: " + str(systemData.samplerate)
			print "iViewX Version: " + str(systemData.iV_MajorVersion) + "." + str(systemData.iV_MinorVersion) + "." + str(systemData.iV_Buildnumber)
			print "iViewX API Version: " + str(systemData.API_MajorVersion) + "." + str(systemData.API_MinorVersion) + "." + str(systemData.API_Buildnumber)

			# ---------------------------------------------
			#---- configure and start calibration
			# ---------------------------------------------

			calibrationData = CCalibration(9, 1, 0, 0, 1, 250, 220, 2, 20, b"")

			res = iViewXAPI.iV_SetupCalibration(byref(calibrationData))
			print "iV_SetupCalibration " + str(res)

			while 1:
				res = iViewXAPI.iV_Calibrate()
				print "iV_Calibrate " + str(res)

				res = iViewXAPI.iV_Validate()
				print "iV_Validate " + str(res)

				res = iViewXAPI.iV_GetAccuracy(byref(accuracyData), 1)
				print "iV_GetAccuracy " + str(res)
				print "deviationXLeft " + str(accuracyData.deviationLX) + " deviationYLeft " + str(accuracyData.deviationLY)
				print "deviationXRight " + str(accuracyData.deviationRX) + " deviationYRight " + str(accuracyData.deviationRY)

				res = iViewXAPI.iV_ShowTrackingMonitor()
				print "iV_ShowTrackingMonitor " + str(res)
				
				q = raw_input("Calibration OK? (y/n) ")
				
				if q == "y":
					break
			
		if ET:
			# open window
			w = visual.Window([scx,scy], fullscr = True, allowGUI= False	, monitor='testMonitor', units='pix', color = [205, 205, 205], colorSpace = 'rgb255')			# window

			# define stimuli
			theFix = visual.Rect(w, pos=(-550,160), height = 20, width = 20, lineColor = "black", lineWidth=6)
			theInstruction = visual.TextStim(w, pos = (0, 0), text = '', wrapWidth = 600, height = 28, color = 'black', alignVert= 'center', font = "Courier New") ###this is the instruction text
			theStatement = visual.TextStim(w, text = "", pos = (-550, 160), height = 40, color = "black", alignVert = "top", alignHoriz = "left", 
				wrapWidth = scx - 200, font = "Courier New") ## thisis the statement/main stimlus
			theQ = visual.TextStim(w, text = "", pos = (0, 100), height = 40, color = "black", alignVert = "top", alignHoriz = "center", 
				wrapWidth = scx - 200, font = "Courier New") ## thisis the statement/main stimlus
			theFollowupInstruction = visual.TextStim(w, pos = (0, -150), text = u'Tryck VNSTER fr NEJ eller HGER fr JA', wrapWidth = 750, height = 25, color = 'black', alignVert= 'top', font = "Courier New") ###this is the instruction text


if ET:
	# CLOSE ET CONNECTION AND SAVE DATA!
	path = "d:/Recorded Data/"
	filename='%s_%s' %('HEN_READ_', subjectInfo[u'Nummer'])
	outputfile = path + filename + ".idf"
	res = iViewXAPI.iV_SaveData(str(outputfile), str("HEN"), str("pp"), 1)
	print "iV_SaveData " + str(res)
	print "data saved to: " + outputfile

	iViewXAPI.iV_Disconnect()

# THE END
theInstruction.setText(u'Du r frdig. Tack!')
theInstruction.draw()
w.flip()
core.wait(0.5)
event.waitKeys(keyList = ['escape'])


core.quit()
